<?php
namespace KnitPay\Gateways\Paytm;

use Pronamic\WordPress\Pay\Core\Gateway as Core_Gateway;
use Pronamic\WordPress\Pay\Payments\Payment;
use Pronamic\WordPress\Pay\Payments\PaymentStatus;
use Exception;
use WP_Error;
use paytm\paytmchecksum\PaytmChecksum;

/**
 * Title: Paytm Gateway
 * Copyright: 2020-2021 Knit Pay
 *
 * @author Knit Pay
 * @version 4.9.0
 * @since 4.9.0
 */
class Gateway extends Core_Gateway {


	const NAME = 'paytm';

	/**
	 * Constructs and initializes an Paytm gateway
	 *
	 * @param Config $config
	 *            Config.
	 */
	public function __construct( Config $config ) {
		parent::__construct( $config );

		$this->set_method( self::METHOD_HTML_FORM );

		// Supported features.
		$this->supports = array(
			'payment_status_request',
		);

		$this->api = new API( $config->merchant_key, self::MODE_TEST === $config->mode );
	}

	/**
	 * Get supported payment methods
	 *
	 * @see Core_Gateway::get_supported_payment_methods()
	 */
	public function get_supported_payment_methods() {
		return array(
			PaymentMethods::BANK_TRANSFER,
			PaymentMethods::CREDIT_CARD,
			PaymentMethods::MAESTRO,
			PaymentMethods::PAYTM,
		);
	}

	/**
	 * Get available payment methods.
	 *
	 * @return array<int, string>
	 * @see Core_Gateway::get_available_payment_methods()
	 */
	public function get_available_payment_methods() {
		return $this->get_supported_payment_methods();
	}

	/**
	 * Start.
	 *
	 * @see Core_Gateway::start()
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function start( Payment $payment ) {
		$payment_currency = $payment->get_total_amount()
			->get_currency()
			->get_alphabetic_code();
		if ( isset( $payment_currency ) && 'INR' !== $payment_currency ) {
			$currency_error = 'Paytm only accepts payments in Indian Rupees. If you are a store owner, kindly activate INR currency for ' . $payment->get_source() . ' plugin.';
			$this->error    = new WP_Error( 'paytm_error', $currency_error );
			return;
		}

		$payment->set_transaction_id( $payment->key . '_' . $payment->get_id() );

		$payment->set_action_url( $this->api->get_endpoint() . 'theia/processTransaction' );

	}

	/**
	 * Get output inputs.
	 *
	 * @see Core_Gateway::get_output_fields()
	 *
	 * @param Payment $payment
	 *            Payment.
	 *
	 * @return array
	 */
	public function get_output_fields( Payment $payment ) {
		$merchant_id  = $this->config->merchant_id;
		$merchant_key = $this->config->merchant_key;
		$website      = $this->config->website;

		$customer        = $payment->get_customer();
		$billing_address = $payment->get_billing_address();

		$cust_id = $customer->get_email();
		if ( empty( $cust_id ) ) {
			$cust_id = 'CUST_' . $payment->get_id();
		}

		$paramList = array();

		// Create an array having all required parameters for creating checksum.
		$paramList['MID']              = $merchant_id;
		$paramList['ORDER_ID']         = $payment->get_transaction_id();
		$paramList['CUST_ID']          = $cust_id;
		$paramList['INDUSTRY_TYPE_ID'] = 'Retail';
		$paramList['CHANNEL_ID']       = 'WEB';
		$paramList['TXN_AMOUNT']       = $payment->get_total_amount()->format();
		$paramList['WEBSITE']          = $website;

		$paramList['CALLBACK_URL'] = $payment->get_return_url();
		$paramList['MSISDN']       = $billing_address->get_phone();
		$paramList['EMAIL']        = $customer->get_email();
		 /*
		  TODO
		  * $paramList["VERIFIED_BY"] = "EMAIL"; //
		 $paramList["IS_USER_VERIFIED"] = "YES"; // */

		// Here checksum string will return by getChecksumFromArray() function.
		$paramList['CHECKSUMHASH'] = PaytmChecksum::generateSignature( $paramList, $merchant_key );

		return $paramList;

	}

	/**
	 * Update status of the specified payment.
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function update_status( Payment $payment ) {
		if ( PaymentStatus::SUCCESS === $payment->get_status() ) {
			return;
		}

		$merchant_id  = $this->config->merchant_id;
		$merchant_key = $this->config->merchant_key;

		// Create an array having all required parameters for status query.
		$requestParamList = array(
			'body' => array(
				'mid'     => $merchant_id,
				'orderId' => $payment->get_transaction_id(),
			),
		);

		$requestParamList['head']['signature'] = PaytmChecksum::generateSignature( wp_json_encode( $requestParamList['body'] ), $merchant_key );

		try {
			$transaction_status = $this->api->get_transaction_status( $requestParamList );
		} catch ( Exception $e ) {
			$this->error = new WP_Error( 'paytm_error', $e->getMessage() );
			return;
		}

		if ( $transaction_status->orderId !== $payment->get_transaction_id() ) {
			return;
		}

		$note = '<strong>Paytm Parameters:</strong>';
		if ( isset( $transaction_status->txnId ) ) {
			$note .= '<br>Paytm txnId: ' . $transaction_status->txnId;
		}
		if ( isset( $transaction_status->bankTxnId ) ) {
			$note .= '<br>Paytm bankTxnId: ' . $transaction_status->bankTxnId;
		}
		$note .= '<br>Paytm orderId: ' . $transaction_status->orderId;
		$note .= '<br>Status: ' . $transaction_status->resultInfo->resultStatus;
		$note .= '<br>Message: ' . $transaction_status->resultInfo->resultMsg;

		$payment->add_note( $note );
		$payment->set_status( Statuses::transform( $transaction_status->resultInfo->resultStatus ) );

		if ( PaymentStatus::SUCCESS === $payment->get_status() ) {
			$payment->set_transaction_id( $transaction_status->txnId );
		}
	}
}
